
#include "nr.h"
#include "nrutil.h"

REAL q,kx0,ky,kz,cs,W,vax,vay0,vaz,vadk;
int nrhs;

void derivs(REAL x,REAL y[],REAL dydx[])
{
	REAL vxr,vxi,vyr,vyi,vzr,vzi,
		rr,ri,bxr,bxi,byr,byi,bzr,bzi ;
	REAL kx,vay ;
	REAL kdvr,kdvi,vadbr,vadbi ;

	nrhs++;
	
	rr = y[1] ;
	ri = y[2] ;
	vxr = y[3] ;
	vxi = y[4] ;
	vyr = y[5] ;
	vyi = y[6] ;
	vzr = y[7] ;
	vzi = y[8] ;
	bxr = y[9] ;
	bxi = y[10] ;
	byr = y[11] ;
	byi = y[12] ;
	bzr = y[13] ;
	bzi = y[14] ;

	kx = kx0 + q*W*ky*x ;
	vay = vay0 - q*W*vax*x ;

	vadbr = vax*bxr + vay*byr + vaz*bzr ;
	vadbi = vax*bxi + vay*byi + vaz*bzi ;

	kdvr = kx*vxr + ky*vyr + kz*vzr ;
	kdvi = kx*vxi + ky*vyi + kz*vzi ;
	
	/* density: continuity equation */
	dydx[1] = kdvi ;
	dydx[2] = -kdvr ;

	/* equations of motion */
	dydx[3] = 2.*W*vyr + kx*(cs*cs*ri + vadbi) - vadk*bxi ;
	dydx[4] = 2.*W*vyi - kx*(cs*cs*rr + vadbr) + vadk*bxr ;
	dydx[5] = (q - 2.)*W*vxr + ky*(cs*cs*ri + vadbi) - vadk*byi ;
	dydx[6] = (q - 2.)*W*vxi - ky*(cs*cs*rr + vadbr) + vadk*byr ;
	dydx[7] = kz*(cs*cs*ri + vadbi) - vadk*bzi ;
	dydx[8] = -kz*(cs*cs*rr + vadbr) + vadk*bzr ;

	/* induction equation */
	dydx[9] = -vadk*vxi + vax*kdvi ;
	dydx[10] = vadk*vxr - vax*kdvr ;
	dydx[11] = -vadk*vyi + vay*kdvi - q*W*bxr ;
	dydx[12] = vadk*vyr - vay*kdvr - q*W*bxi ;
	dydx[13] = -vadk*vzi + vaz*kdvi ;
	dydx[14] = vadk*vzr - vaz*kdvr ;
}


void jacobn(REAL x, REAL y[], REAL dfdx[], REAL **dfdy, int n)
{
	int i;
	REAL vxr,vxi,vyr,vyi,vzr,vzi,
		rr,ri,bxr,bxi,byr,byi,bzr,bzi ;
	REAL kx,vay,kxp,vayp;
	REAL kdvr,kdvi,vadbr,vadbi ;
	
	rr = y[1] ;
	ri = y[2] ;
	vxr = y[3] ;
	vxi = y[4] ;
	vyr = y[5] ;
	vyi = y[6] ;
	vzr = y[7] ;
	vzi = y[8] ;
	bxr = y[9] ;
	bxi = y[10] ;
	byr = y[11] ;
	byi = y[12] ;
	bzr = y[13] ;
	bzi = y[14] ;
	
	kx = kx0 + q*W*ky*x ;
	vay = vay0 - q*W*vax*x ;
	kxp=q*W*ky;
	vayp=-q*W*vax;
	
	vadbr = vax*bxr + vay*byr + vaz*bzr ;
	vadbi = vax*bxi + vay*byi + vaz*bzi ;

	kdvr = kx*vxr + ky*vyr + kz*vzr ;
	kdvi = kx*vxi + ky*vyi + kz*vzi ;
	
	dfdx[1]=kxp*vxi;
	dfdx[2]=-kxp*vxr;
	dfdx[3]=kxp*(cs*cs*ri + vadbi) + kx*vayp*byi;
	dfdx[4]=-kxp*(cs*cs*rr + vadbr) - kx*vayp*byr;
	dfdx[5]=ky*vayp*byi;
	dfdx[6]=-ky*vayp*byr;
	dfdx[7]=kz*vayp*byi;
	dfdx[8]=-kz*vayp*byr;
	dfdx[9]=vax*kxp*vxi;
	dfdx[10]=-vax*kxp*vxr;
	dfdx[11]=vay*kxp*vxi + vayp*kdvi;
	dfdx[12]=-vay*kxp*vxr - vayp*kdvr;
	dfdx[13]=vaz*kxp*vxi;
	dfdx[14]=-vaz*kxp*vxr;
	
	dfdy[1][1]=dfdy[1][2]=dfdy[1][9]=dfdy[1][10]=dfdy[1][11]=dfdy[1][12]=dfdy[1][13]=dfdy[1][14]=0.;
	dfdy[1][3]=0.;
	dfdy[1][4]=kx;
	dfdy[1][5]=0.;
	dfdy[1][6]=ky;
	dfdy[1][7]=0.;
	dfdy[1][8]=kz;
	
	dfdy[2][1]=dfdy[2][2]=dfdy[2][9]=dfdy[2][10]=dfdy[2][11]=dfdy[2][12]=dfdy[2][13]=dfdy[2][14]=0.;
	dfdy[2][3]=-kx;
	dfdy[2][4]=0.;
	dfdy[2][5]=-ky;
	dfdy[2][6]=0.;
	dfdy[2][7]=-kz;
	dfdy[2][8]=0.;
	
	dfdy[3][1]=0.;
	dfdy[3][2]=kx*cs*cs;
	dfdy[3][3]=dfdy[3][4]=dfdy[3][7]=dfdy[3][8]=0.;
	dfdy[3][5]=2.*W;
	dfdy[3][6]=0.;
	dfdy[3][9]=0.;
	dfdy[3][10]=kx*vax-vadk;
	dfdy[3][11]=0.;
	dfdy[3][12]=kx*vay;
	dfdy[3][13]=0.;
	dfdy[3][14]=kx*vaz;
	
	dfdy[4][1]=-kx*cs*cs;
	dfdy[4][2]=0.;
	dfdy[4][3]=dfdy[4][4]=dfdy[4][7]=dfdy[4][8]=0.;
	dfdy[4][5]=0.;
	dfdy[4][6]=2.*W;
	dfdy[4][9]=-kx*vax+vadk;
	dfdy[4][10]=0.;
	dfdy[4][11]=-kx*vay;
	dfdy[4][12]=0.;
	dfdy[4][13]=-kx*vaz;
	dfdy[4][14]=0.;
	
	dfdy[5][1]=0.;
	dfdy[5][2]=ky*cs*cs;
	dfdy[5][3]=(q-2.)*W;
	dfdy[5][4]=0.;
	dfdy[5][5]=dfdy[5][6]=dfdy[5][7]=dfdy[5][8]=0.;
	dfdy[5][9]=0.;
	dfdy[5][10]=ky*vax;
	dfdy[5][11]=0.;
	dfdy[5][12]=ky*vay-vadk;
	dfdy[5][13]=0.;
	dfdy[5][14]=ky*vaz;
	
	dfdy[6][1]=-ky*cs*cs;
	dfdy[6][2]=0.;
	dfdy[6][3]=0.;
	dfdy[6][4]=(q-2.)*W;
	dfdy[6][5]=dfdy[6][6]=dfdy[6][7]=dfdy[6][8]=0.;
	dfdy[6][9]=-ky*vax;
	dfdy[6][10]=0.;
	dfdy[6][11]=-ky*vay+vadk;
	dfdy[6][12]=0.;
	dfdy[6][13]=-ky*vaz;
	dfdy[6][14]=0.;
	
	dfdy[7][1]=0.;
	dfdy[7][2]=kz*cs*cs;
	dfdy[7][3]=dfdy[7][4]=dfdy[7][5]=dfdy[7][6]=dfdy[7][7]=dfdy[7][8]=0.;
	dfdy[7][9]=0.;
	dfdy[7][10]=kz*vax;
	dfdy[7][11]=0.;
	dfdy[7][12]=kz*vay;
	dfdy[7][13]=0.;
	dfdy[7][14]=kz*vaz-vadk;
	
	dfdy[8][1]=-kz*cs*cs;
	dfdy[8][2]=0.;
	dfdy[8][3]=dfdy[8][4]=dfdy[8][5]=dfdy[8][6]=dfdy[8][7]=dfdy[8][8]=0.;
	dfdy[8][9]=-kz*vax;
	dfdy[8][10]=0.;
	dfdy[8][11]=-kz*vay;
	dfdy[8][12]=0.;
	dfdy[8][13]=-kz*vaz+vadk;
	dfdy[8][14]=0.;
	
	dfdy[9][1]=dfdy[9][2]=0.;
	dfdy[9][3]=0.;
	dfdy[9][4]=-vadk+vax*kx;
	dfdy[9][5]=0.;
	dfdy[9][6]=vax*ky;
	dfdy[9][7]=0.;
	dfdy[9][8]=vax*kz;
	dfdy[9][9]=dfdy[9][10]=dfdy[9][11]=dfdy[9][12]=dfdy[9][13]=dfdy[9][14]=0.;
		
	dfdy[10][1]=dfdy[10][2]=0.;
	dfdy[10][3]=vadk-vax*kx;
	dfdy[10][4]=0.;
	dfdy[10][5]=-vax*ky;
	dfdy[10][6]=0.;
	dfdy[10][7]=-vax*kz;
	dfdy[10][8]=0.;
	dfdy[10][9]=dfdy[10][10]=dfdy[10][11]=dfdy[10][12]=dfdy[10][13]=dfdy[10][14]=0.;
	
	dfdy[11][1]=dfdy[11][2]=0.;
	dfdy[11][3]=0.;
	dfdy[11][4]=vay*kx;
	dfdy[11][5]=0.;
	dfdy[11][6]=-vadk+vay*ky;
	dfdy[11][7]=0.;
	dfdy[11][8]=vay*kz;
	dfdy[11][9]=-q*W;
	dfdy[11][10]=0.;
	dfdy[11][11]=dfdy[11][12]=dfdy[11][13]=dfdy[11][14]=0.;
			
	dfdy[12][1]=dfdy[12][2]=0.;
	dfdy[12][3]=-vay*kx;
	dfdy[12][4]=0.;
	dfdy[12][5]=vadk-vay*ky;
	dfdy[12][6]=0.;
	dfdy[12][7]=-vay*kz;
	dfdy[12][8]=0.;
	dfdy[12][9]=0.;
	dfdy[12][10]=-q*W;
	dfdy[12][11]=dfdy[12][12]=dfdy[12][13]=dfdy[12][14]=0.;

	dfdy[13][1]=dfdy[13][2]=0.;
	dfdy[13][3]=0.;
	dfdy[13][4]=vaz*kx;
	dfdy[13][5]=0.;
	dfdy[13][6]=vaz*ky;
	dfdy[13][7]=0.;
	dfdy[13][8]=-vadk+vaz*kz;
	dfdy[13][9]=dfdy[13][10]=dfdy[13][11]=dfdy[13][12]=dfdy[13][13]=dfdy[13][14]=0.;
		
	dfdy[14][1]=dfdy[14][2]=0.;
	dfdy[14][3]=-vaz*kx;
	dfdy[14][4]=0.;
	dfdy[14][5]=-vaz*ky;
	dfdy[14][6]=0.;
	dfdy[14][7]=vadk-vaz*kz;
	dfdy[14][8]=0.;
	dfdy[14][9]=dfdy[14][10]=dfdy[14][11]=dfdy[14][12]=dfdy[14][13]=dfdy[14][14]=0.;
}

/* (C) Copr. 1986-92 Numerical Recipes Software vO(1Z712,$. */
