#include <math.h>
#define NRANSI
#include "nrutil.h"
#define MAXSTP 1000000
#define TINY 1.0e-30

extern int kmax,kount;
extern REAL *xp,**yp,dxsav;

void odeint(REAL ystart[], int nvar, REAL x1, REAL x2, REAL eps, REAL h1,
	REAL hmin, int *nok, int *nbad,
	void (*derivs)(REAL, REAL [], REAL []),
	void (*rkqs)(REAL [], REAL [], int, REAL *, REAL, REAL, REAL [],
	REAL *, REAL *, void (*)(REAL, REAL [], REAL [])))
{
	int nstp,i;
	REAL xsav,hnext,hdid,h;
	REAL *yscal,x,*y,*dydx;

#if FLOAT
	yscal=vector(1,nvar);
	y=vector(1,nvar);
	dydx=vector(1,nvar);
#else
	yscal=dvector(1,nvar);
	y=dvector(1,nvar);
	dydx=dvector(1,nvar);
#endif
	x=x1;
	h=SIGN(h1,x2-x1);
	*nok = (*nbad) = kount = 0;
	for (i=1;i<=nvar;i++) y[i]=ystart[i];
	if (kmax > 0) xsav=x-dxsav*2.0;
	for (nstp=1;nstp<=MAXSTP;nstp++) {
		(*derivs)(x,y,dydx);
		for (i=1;i<=nvar;i++)
			yscal[i]=fabs(y[i])+fabs(dydx[i]*h)+TINY;
		if (kmax > 0 && kount < kmax-1 && fabs(x-xsav) > fabs(dxsav)) {
			xp[++kount]=x;
			for (i=1;i<=nvar;i++) yp[i][kount]=y[i];
			xsav=x;
		}
		if ((x+h-x2)*(x+h-x1) > 0.0) h=x2-x;
		(*rkqs)(y,dydx,nvar,&x,h,eps,yscal,&hdid,&hnext,derivs);
		if (hdid == h) ++(*nok); else ++(*nbad);
		if ((x-x2)*(x2-x1) >= 0.0) {
			for (i=1;i<=nvar;i++) ystart[i]=y[i];
			if (kmax) {
				xp[++kount]=x;
				for (i=1;i<=nvar;i++) yp[i][kount]=y[i];
			}
#if FLOAT
			free_vector(dydx,1,nvar);
			free_vector(y,1,nvar);
			free_vector(yscal,1,nvar);
#else
			free_dvector(dydx,1,nvar);
			free_dvector(y,1,nvar);
			free_dvector(yscal,1,nvar);
#endif
			return;
		}
		if (fabs(hnext) <= hmin) nrerror("Step size too small in odeint");
		h=hnext;
	}
	nrerror("Too many steps in routine odeint");
}
#undef MAXSTP
#undef TINY
#undef NRANSI
/* (C) Copr. 1986-92 Numerical Recipes Software vO(1Z712,$. */
